function [r, k] = secant ( f, a )

% Secant method for finding root of f "near" an initial guess a
% OUTPUT: r = root found
% OUTPUT: k = No. of iteration required

Nmax       = 1000      ; % Max. # of Iterations allowed
epsilon    = 0.001     ; % Error tolerance
small_step = epsilon/10; % Small step

% Secant method
for k = 1:Nmax

% Find two points 
  x1 = a;               y1 = feval (f, x1);
  x2 = a + small_step;  y2 = feval (f, x2);

% Find b = x-intercept of the (secant) line i.e., the new approximation
% which has no interaction to the "b" in sub-function
  b = line_intercept ( x1, y1, x2, y2 );
      
% Stopping criterion
  if ( abs(b - a) < epsilon )
    r = b; break; % to quit the for-loop
  end

% If not-ready-to-stop yet, use new approximation to repeat
  a = b;
end

if k == Nmax
  disp ('The method does not converge');
end

%---------------------------------------------------------
% Sub-function within function

function [ x_intercept ] = line_intercept (x1, y1, x2, y2)

% Find x_intercept of the line y = m*x +b 
%   that pass through (x1,y1) and (x2,y2)

          m = (y2-y1)/(x2-x1) ;
          b = y1-m*x1         ;
x_intercept = -b/m            ;
